# demo.tcl - fred.

namespace import ::GL::*

menu .menu -tearoff 0

set m .menu.file
menu $m -tearoff 0
.menu add cascade -label "File" -menu $m -underline 0
$m add command -label "Exit" -underline 0 -command "destroy ."

set m .menu.windows
menu $m -tearoff 0
.menu add cascade -label "Windows" -menu $m -underline 0
$m add command -label "Material..." -underline 0 -command materialWindow
$m add command -label "Light..." -underline 0 -command lightWindow
$m add command -label "Light model..." -underline 1 -command lightModelWindow
$m add command -label "Fog..." -underline 0 -command {} -state disabled

set m .menu.windows.lightModel
menu $m -tearoff 0
$m add command -label "Ambient..." -underline 0 -command {}

. configure -menu .menu

proc updateMaterial {pname redraw args} {
    global MATERIAL

    switch -exact -- $pname {
	GL_AMBIENT {
	    set MATERIAL(GL_AMBIENT) [list $MATERIAL(GL_AMBIENT.red) $MATERIAL(GL_AMBIENT.green)\
		    $MATERIAL(GL_AMBIENT.blue) $MATERIAL(GL_AMBIENT.alpha)]
	}
	GL_EMISSION {
	    set MATERIAL(GL_EMISSION) [list $MATERIAL(GL_EMISSION.red) $MATERIAL(GL_EMISSION.green)\
		    $MATERIAL(GL_EMISSION.blue) $MATERIAL(GL_EMISSION.alpha)]
	}
	GL_DIFFUSE {
	    set MATERIAL(GL_DIFFUSE) [list $MATERIAL(GL_DIFFUSE.red) $MATERIAL(GL_DIFFUSE.green)\
		    $MATERIAL(GL_DIFFUSE.blue) $MATERIAL(GL_DIFFUSE.alpha)]
	}
	GL_SPECULAR {
	    set MATERIAL(GL_SPECULAR) [list $MATERIAL(GL_SPECULAR.red) $MATERIAL(GL_SPECULAR.green)\
		    $MATERIAL(GL_SPECULAR.blue) $MATERIAL(GL_SPECULAR.alpha)]
	}
	GL_SHININESS {
	    set MATERIAL(GL_SHININESS) $MATERIAL(GL_SHININESS.exp)
	}
    }
    if {$redraw} {draw; flush}
}

proc updateLight {pname redraw args} {
    global LIGHT

    switch -exact -- $pname {
	GL_AMBIENT {
	    set LIGHT(GL_AMBIENT) [list $LIGHT(GL_AMBIENT.red) $LIGHT(GL_AMBIENT.green)\
		    $LIGHT(GL_AMBIENT.blue) $LIGHT(GL_AMBIENT.alpha)]
	}
	GL_DIFFUSE {
	    set LIGHT(GL_DIFFUSE) [list $LIGHT(GL_DIFFUSE.red) $LIGHT(GL_DIFFUSE.green)\
		    $LIGHT(GL_DIFFUSE.blue) $LIGHT(GL_DIFFUSE.alpha)]
	}
	GL_SPECULAR {
	    set LIGHT(GL_SPECULAR) [list $LIGHT(GL_SPECULAR.red) $LIGHT(GL_SPECULAR.green)\
		    $LIGHT(GL_SPECULAR.blue) $LIGHT(GL_SPECULAR.alpha)]
	}
    }
    if {$redraw} {draw; flush}   
}

proc updateLightModel {pname redraw args} {
    global LIGHTMODEL

    switch -exact -- $pname {
	GL_LIGHT_MODEL_AMBIENT {
	    set LIGHTMODEL(GL_LIGHT_MODEL_AMBIENT)\
		    [list $LIGHTMODEL(GL_LIGHT_MODEL_AMBIENT.red)\
		    $LIGHTMODEL(GL_LIGHT_MODEL_AMBIENT.green)\
		    $LIGHTMODEL(GL_LIGHT_MODEL_AMBIENT.blue)\
		    $LIGHTMODEL(GL_LIGHT_MODEL_AMBIENT.alpha)]
	}
    }
    if {$redraw} {draw; flush}   
}

proc init {} {
    global BG MATERIAL LIGHT LIGHTMODEL

    glEnable $GL::GL_DEPTH_TEST
    glDepthFunc $GL::GL_LESS
    glShadeModel $GL::GL_SMOOTH
    glEnable $GL::GL_BLEND
    glBlendFunc $GL::GL_SRC_ALPHA $GL::GL_ONE_MINUS_SRC_ALPHA

    glClearColor 0.0 0.1 0.1 0.0

    glLightfv $GL::GL_LIGHT0 $GL::GL_POSITION {3.0 3.0 2.0 0.0}
    glEnable $GL::GL_LIGHTING
    glEnable $GL::GL_LIGHT0

    glLightModelfv $GL::GL_LIGHT_MODEL_LOCAL_VIEWER 0.0

    set MATERIAL(GL_AMBIENT.red) 0.2
    set MATERIAL(GL_AMBIENT.green) 0.2
    set MATERIAL(GL_AMBIENT.blue) 0.2
    set MATERIAL(GL_AMBIENT.alpha) 1.0
    set MATERIAL(GL_EMISSION.red) 0.0
    set MATERIAL(GL_EMISSION.green) 0.0
    set MATERIAL(GL_EMISSION.blue) 0.0
    set MATERIAL(GL_EMISSION.alpha) 1.0
    set MATERIAL(GL_DIFFUSE.red) 0.8
    set MATERIAL(GL_DIFFUSE.green) 0.8
    set MATERIAL(GL_DIFFUSE.blue) 0.8
    set MATERIAL(GL_DIFFUSE.alpha) 1.0
    set MATERIAL(GL_SPECULAR.red) 0.0
    set MATERIAL(GL_SPECULAR.green) 0.0
    set MATERIAL(GL_SPECULAR.blue) 0.0
    set MATERIAL(GL_SPECULAR.alpha) 1.0
    set MATERIAL(GL_SHININESS.exp) 128.0
    updateMaterial GL_AMBIENT 0
    updateMaterial GL_EMISSION 0
    updateMaterial GL_DIFFUSE 0
    updateMaterial GL_SPECULAR 0
    updateMaterial GL_SHININESS 0

    set LIGHT(GL_AMBIENT.red) 0.0
    set LIGHT(GL_AMBIENT.green) 0.0
    set LIGHT(GL_AMBIENT.blue) 0.0
    set LIGHT(GL_AMBIENT.alpha) 1.0
    set LIGHT(GL_DIFFUSE.red) 1.0
    set LIGHT(GL_DIFFUSE.green) 1.0
    set LIGHT(GL_DIFFUSE.blue) 1.0
    set LIGHT(GL_DIFFUSE.alpha) 1.0
    set LIGHT(GL_SPECULAR.red) 1.0
    set LIGHT(GL_SPECULAR.green) 1.0
    set LIGHT(GL_SPECULAR.blue) 1.0
    set LIGHT(GL_SPECULAR.alpha) 1.0
    updateLight GL_AMBIENT 0
    updateLight GL_DIFFUSE 0
    updateLight GL_SPECULAR 0

    set LIGHTMODEL(GL_LIGHT_MODEL_AMBIENT.red) 0.4
    set LIGHTMODEL(GL_LIGHT_MODEL_AMBIENT.green) 0.4
    set LIGHTMODEL(GL_LIGHT_MODEL_AMBIENT.blue) 0.4
    set LIGHTMODEL(GL_LIGHT_MODEL_AMBIENT.alpha) 1.0
    updateLightModel GL_LIGHT_MODEL_AMBIENT 0

    # make texture
    set checkImageWidth 64
    set checkImageHeight 64
    set tex {}
    for {set i 0} {$i < $checkImageWidth} {incr i} {
	for {set j 0} {$j < $checkImageHeight} {incr j} {
	    set c [expr ((($i & 8) == 0) ^ (($j & 8) == 0)) * 255]
	    lappend tex $c $c $c 255
	}
    }
    set tex [binary format c* $tex]
    glPixelStorei $GL::GL_UNPACK_ALIGNMENT 1
    glTexImage2D -b -var $GL::GL_TEXTURE_2D 0 4\
	    $checkImageWidth $checkImageHeight\
	    0 $GL::GL_RGBA $GL::GL_UNSIGNED_BYTE tex
    glTexParameteri $GL::GL_TEXTURE_2D $GL::GL_TEXTURE_WRAP_S $GL::GL_CLAMP
    glTexParameteri $GL::GL_TEXTURE_2D $GL::GL_TEXTURE_WRAP_T $GL::GL_CLAMP
    glTexParameteri $GL::GL_TEXTURE_2D $GL::GL_TEXTURE_MAG_FILTER $GL::GL_LINEAR
    glTexParameteri $GL::GL_TEXTURE_2D $GL::GL_TEXTURE_MIN_FILTER $GL::GL_LINEAR
    glTexEnvi $GL::GL_TEXTURE_ENV $GL::GL_TEXTURE_ENV_MODE $GL::GL_MODULATE

    set BG 1
    glNewList $BG $GL::GL_COMPILE
    glMaterialfv $GL::GL_FRONT_AND_BACK $GL::GL_DIFFUSE {0.8 0.8 0.8 1.0}
    glMaterialfv $GL::GL_FRONT_AND_BACK $GL::GL_AMBIENT {0.2 0.2 0.2 1.0}
    glMaterialfv $GL::GL_FRONT_AND_BACK $GL::GL_EMISSION {0.0 0.0 0.0 1.0}
    glMaterialfv $GL::GL_FRONT_AND_BACK $GL::GL_SPECULAR {0.0 0.0 0.0 1.0}
    glBegin $GL::GL_QUADS
    glNormal3f 0.0 0.0 1.0
    glTexCoord2f 1.0 1.0
    glVertex3f 1.5 1.5 -5.0
    glTexCoord2f 0.0 1.0
    glVertex3f -1.5 1.5 -5.0
    glTexCoord2f 0.0 0.0
    glVertex3f -1.5 -1.5 -5.0
    glTexCoord2f 1.0 0.0
    glVertex3f 1.5 -1.5 -5.0
    glEnd
    glEndList
}

proc sphere radius {
    set q [gluNewQuadric]
    gluSphere $q $radius 32 32
    gluDeleteQuadric $q
}

proc draw {} {
    global BG MATERIAL LIGHT LIGHTMODEL
    
    glClear [expr $GL::GL_COLOR_BUFFER_BIT | $GL::GL_DEPTH_BUFFER_BIT]
    
    glLightModelfv $GL::GL_LIGHT_MODEL_AMBIENT $LIGHTMODEL(GL_LIGHT_MODEL_AMBIENT)
    
    glLightfv $GL::GL_LIGHT0 $GL::GL_DIFFUSE $LIGHT(GL_DIFFUSE)
    glLightfv $GL::GL_LIGHT0 $GL::GL_AMBIENT $LIGHT(GL_AMBIENT)
    glLightfv $GL::GL_LIGHT0 $GL::GL_SPECULAR $LIGHT(GL_SPECULAR)
    
    # draw background
    glEnable $GL::GL_TEXTURE_2D
    glCallList $BG
    glDisable $GL::GL_TEXTURE_2D
    
    #draw sphere
    glMaterialfv $GL::GL_FRONT_AND_BACK $GL::GL_DIFFUSE $MATERIAL(GL_DIFFUSE)
    glMaterialfv $GL::GL_FRONT_AND_BACK $GL::GL_AMBIENT $MATERIAL(GL_AMBIENT)
    glMaterialfv $GL::GL_FRONT_AND_BACK $GL::GL_EMISSION $MATERIAL(GL_EMISSION)
    glMaterialfv $GL::GL_FRONT_AND_BACK $GL::GL_SPECULAR $MATERIAL(GL_SPECULAR)
    glMaterialfv $GL::GL_FRONT_AND_BACK $GL::GL_SHININESS $MATERIAL(GL_SHININESS)
    glPushMatrix
    glTranslatef 0 0 -3
    sphere 1
    glPopMatrix
}

proc makeMaterialWindow {name pname params} {
    global MATERIAL

    set top [frame .materialWindow.[string tolower $name]\
	    -relief groove -bd 1]
    set l [label $top.title -text $name -anchor w]
    pack $l -anchor w
    foreach p $params {
	set f [frame $top.f[lindex $p 0]]
	set l [label $f.l[lindex $p 0] -text [lindex $p 1]]
	pack $l -side left -anchor se
	set s [scale $f.s[lindex $p 0]\
		-from [lindex $p 2]\
		-to [lindex $p 3]\
		-resolution [lindex $p 4]\
		-orient horizontal\
		-variable MATERIAL($pname.[lindex $p 0])\
		-command "updateMaterial $pname 1"]
	pack $s -side right -expand 1 -fill x
	pack $f -anchor nw
    }
    return $top
}

proc materialWindow {} {
    set top .materialWindow
    if {![winfo exists $top]} {
	toplevel $top
	#wm transient $top .
	wm title $top "Material"

	set w [makeMaterialWindow Ambient GL_AMBIENT [list\
		[list red R 0.0 1.0 0.025] [list green G 0.0 1.0 0.025]\
		[list blue B 0.0 1.0 0.025] [list alpha A 0.0 1.0 0.025]]]
	pack $w -side left -anchor nw -padx 2 -pady 2 -expand 1 -fill both

	set w [makeMaterialWindow Diffuse GL_DIFFUSE [list\
		[list red R 0.0 1.0 0.025] [list green G 0.0 1.0 0.025]\
		[list blue B 0.0 1.0 0.025] [list alpha A 0.0 1.0 0.025]]]
	pack $w -side left -anchor nw -padx 2 -pady 2 -expand 1 -fill both

	set w [makeMaterialWindow Specular GL_SPECULAR [list\
		[list red R 0.0 1.0 0.025] [list green G 0.0 1.0 0.025]\
		[list blue B 0.0 1.0 0.025] [list alpha A 0.0 1.0 0.025]]]
	pack $w -side left -anchor nw -padx 2 -pady 2 -expand 1 -fill both

	set w [makeMaterialWindow Emission GL_EMISSION [list\
		[list red R 0.0 1.0 0.025] [list green G 0.0 1.0 0.025]\
		[list blue B 0.0 1.0 0.025] [list alpha A 0.0 1.0 0.025]]]
	pack $w -side left -anchor nw -padx 2 -pady 2 -expand 1 -fill both

	set w [makeMaterialWindow Shininess GL_SHININESS [list\
		[list exp {} 0 128 0.5]]]
	pack $w -side left -anchor nw -padx 2 -pady 2 -expand 1 -fill both
    }
    wm resizable $top 0 0
}

proc makeLightWindow {name pname params} {
    global LIGHT

    set top [frame .lightWindow.[string tolower $name]\
	    -relief groove -bd 1]
    set l [label $top.title -text $name -anchor w]
    pack $l -anchor w
    foreach p $params {
	set f [frame $top.f[lindex $p 0]]
	set l [label $f.l[lindex $p 0] -text [lindex $p 1]]
	pack $l -side left -anchor se
	set s [scale $f.s[lindex $p 0]\
		-from [lindex $p 2]\
		-to [lindex $p 3]\
		-resolution [lindex $p 4]\
		-orient horizontal\
		-variable LIGHT($pname.[lindex $p 0])\
		-command "updateLight $pname 1"]
	pack $s -side right -expand 1 -fill x
	pack $f -anchor nw
    }
    return $top
}

proc lightWindow {} {
    set top .lightWindow
    if {![winfo exists $top]} {
	toplevel $top
	#wm transient $top .
	wm title $top "Light"

	set w [makeLightWindow Ambient GL_AMBIENT [list\
		[list red R 0.0 1.0 0.025] [list green G 0.0 1.0 0.025]\
		[list blue B 0.0 1.0 0.025] [list alpha A 0.0 1.0 0.025]]]
	pack $w -side left -anchor nw -padx 2 -pady 2

	set w [makeLightWindow Diffuse GL_DIFFUSE [list\
		[list red R 0.0 1.0 0.025] [list green G 0.0 1.0 0.025]\
		[list blue B 0.0 1.0 0.025] [list alpha A 0.0 1.0 0.025]]]
	pack $w -side left -anchor nw -padx 2 -pady 2

	set w [makeLightWindow Specular GL_SPECULAR [list\
		[list red R 0.0 1.0 0.025] [list green G 0.0 1.0 0.025]\
		[list blue B 0.0 1.0 0.025] [list alpha A 0.0 1.0 0.025]]]
	pack $w -side left -anchor nw -padx 2 -pady 2
    }
    wm resizable $top 0 0
}

proc makeLightModelWindow {name pname params} {
    global LIGHTMODEL

    set top [frame .lightModelWindow.[string tolower $name]\
	    -relief groove -bd 1]
    set l [label $top.title -text $name -anchor w]
    pack $l -anchor w
    foreach p $params {
	set f [frame $top.f[lindex $p 0]]
	set l [label $f.l[lindex $p 0] -text [lindex $p 1]]
	pack $l -side left -anchor se
	set s [scale $f.s[lindex $p 0]\
		-from [lindex $p 2]\
		-to [lindex $p 3]\
		-resolution [lindex $p 4]\
		-orient horizontal\
		-variable LIGHTMODEL($pname.[lindex $p 0])\
		-command "updateLightModel $pname 1"]
	pack $s -side right -expand 1 -fill x
	pack $f -anchor nw
    }
    return $top
}

proc lightModelWindow {} {
    set top .lightModelWindow
    if {![winfo exists $top]} {
	toplevel $top
	#wm transient $top .
	wm title $top "LightModel"

	set w [makeLightModelWindow Ambient GL_LIGHT_MODEL_AMBIENT [list\
		[list red R 0.0 1.0 0.025] [list green G 0.0 1.0 0.025]\
		[list blue B 0.0 1.0 0.025] [list alpha A 0.0 1.0 0.025]]]
	pack $w -side left -anchor nw -padx 2 -pady 2
    }
    wm resizable $top 0 0
}

proc flush {} {
	.gl swapbuffers
}

tom .gl -doublebuffer 1 -depth 1 -width 300 -height 300

init

bind .gl <Configure> {
    glViewport 0 0 %w %h
    glMatrixMode $GL::GL_PROJECTION
    glLoadIdentity
    glOrtho -1.5 1.5 -1.5 1.5 -10.0 10.0
    glMatrixMode $GL::GL_MODELVIEW
}

bind .gl <Expose> {
    draw
    flush
}

pack .gl -fill both -expand 1
