# material.tcl - fred.

namespace import ::GL::*

proc init {} {
    set ambient  {0.0 0.0 0.0 1.0}
    set diffuse {1.0 1.0 1.0 1.0}
    set position {0.0 3.0 2.0 0.0}
    set lmodel_ambient {0.4 0.4 0.4 1.0}
    set local_view {0.0}
            
    glEnable $GL::GL_DEPTH_TEST
    glDepthFunc $GL::GL_LESS
            
    glLightfv $GL::GL_LIGHT0 $GL::GL_AMBIENT $ambient
    glLightfv $GL::GL_LIGHT0 $GL::GL_DIFFUSE $diffuse
    glLightfv $GL::GL_LIGHT0 $GL::GL_POSITION $position
    #glLightModelfv $GL::GL_LIGHT_MODEL_AMBIENT $lmodel_ambient
    #glLightModelfv $GL::GL_LIGHT_MODEL_LOCAL_VIEWER $local_view

    glEnable $GL::GL_LIGHTING;
    glEnable $GL::GL_LIGHT0;

    glClearColor 0.0 0.1 0.1 0.0
}

proc sphere radius {
    set q [gluNewQuadric]
    gluSphere $q $radius 16 16
    gluDeleteQuadric $q
}

proc draw {} {
    set no_mat {0.0 0.0 0.0 1.0}
    set mat_ambient {0.7 0.7 0.7 1.0}
    set mat_ambient_color {0.8 0.8 0.2 1.0}
    set mat_diffuse {0.1 0.5 0.8 1.0}
    set mat_specular {1.0 1.0 1.0 1.0}
    set no_shininess {0.0}
    set low_shininess {5.0}
    set high_shininess {100.0}
    set mat_emission {0.3 0.2 0.2 0.0}
	
    glClear [expr $GL::GL_COLOR_BUFFER_BIT | $GL::GL_DEPTH_BUFFER_BIT]
            
    #  draw sphere in first row, first column
    #  diffuse reflection only no ambient or specular  
    glPushMatrix
    glTranslated -3.75 3.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $no_mat
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $no_mat
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $no_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $no_mat
    sphere 1.0          
    glPopMatrix
    
    #  draw sphere in first row, second column
    #  diffuse and specular reflection low shininess no ambient
    glPushMatrix
    glTranslated -1.25 3.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $no_mat
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $mat_specular
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $low_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $no_mat
    sphere 1.0            
    glPopMatrix
            
    #  draw sphere in first row, third column
    #  diffuse and specular reflection high shininess no ambient
    glPushMatrix
    glTranslated 1.25 3.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $no_mat
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $mat_specular
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $high_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $no_mat
    sphere 1.0            
    glPopMatrix
    
    #  draw sphere in first row, fourth column
    #  diffuse reflection emission no ambient or specular reflection
    glPushMatrix
    glTranslated 3.75 3.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $no_mat
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $no_mat
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $no_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $mat_emission
    sphere 1.0            
    glPopMatrix
            
    #  draw sphere in second row, first column
    #  ambient and diffuse reflection no specular  
    glPushMatrix
    glTranslated -3.75 0.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $mat_ambient
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $no_mat
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $no_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $no_mat
    sphere 1.0            
    glPopMatrix
    
    #  draw sphere in second row, second column
    #  ambient, diffuse and specular reflection low shininess
    glPushMatrix
    glTranslated -1.25 0.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $mat_ambient
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $mat_specular
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $low_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $no_mat
    sphere 1.0            
    glPopMatrix
            
    #  draw sphere in second row, third column
    #  ambient, diffuse and specular reflection high shininess
    glPushMatrix
    glTranslated 1.25 0.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $mat_ambient
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $mat_specular
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $high_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $no_mat
    sphere 1.0            
    glPopMatrix
            
    #  draw sphere in second row, fourth column
    #  ambient and diffuse reflection emission no specular
    glPushMatrix
    glTranslated 3.75 0.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $mat_ambient
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $no_mat
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $no_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $mat_emission
    sphere 1.0            
    glPopMatrix
            
    #  draw sphere in third row, first column
    #  colored ambient and diffuse reflection no specular  
    glPushMatrix
    glTranslated -3.75 -3.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $mat_ambient_color
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $no_mat
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $no_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $no_mat
    sphere 1.0            
    glPopMatrix
            
    #  draw sphere in third row, second column
    #  colored ambient, diffuse and specular reflection low shininess
    glPushMatrix
    glTranslated -1.25 -3.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $mat_ambient_color
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $mat_specular
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $low_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $no_mat
    sphere 1.0            
    glPopMatrix
            
    #  draw sphere in third row, third column
    #  colored ambient, diffuse and specular reflection high shininess
    glPushMatrix
    glTranslated 1.25 -3.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $mat_ambient_color
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $mat_specular
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $high_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $no_mat
    sphere 1.0            
    glPopMatrix
            
    #  draw sphere in third row, fourth column
    #  colored ambient and diffuse reflection emission no specular
    glPushMatrix
    glTranslated 3.75 -3.0 0.0
    glMaterialfv $GL::GL_FRONT $GL::GL_AMBIENT $mat_ambient_color
    glMaterialfv $GL::GL_FRONT $GL::GL_DIFFUSE $mat_diffuse
    glMaterialfv $GL::GL_FRONT $GL::GL_SPECULAR $no_mat
    glMaterialfv $GL::GL_FRONT $GL::GL_SHININESS $no_shininess
    glMaterialfv $GL::GL_FRONT $GL::GL_EMISSION $mat_emission
    sphere 1.0            
    glPopMatrix
    
    glFlush
}

tom .gl -depth 1 -width 300 -height 225

init

bind .gl <Configure> {
    glViewport 0 0 %w %h
    glMatrixMode $GL::GL_PROJECTION
    glLoadIdentity
    if {%w <= (%h * 2)} {
	glOrtho -6.0 6.0 [expr (-3.0 * %h * 2) / %w] [expr (3.0 * %h * 2) / %w] -10.0 10.0
    } else {
	glOrtho [expr (-6.0 * %w) / (%h * 2)] [expr (6.0 * %w) / (%h * 2)] -3.0 3.0 -10.0 10.0
    }
    glMatrixMode $GL::GL_MODELVIEW
}

bind .gl <Expose> {draw}

pack .gl -fill both -expand 1
