# select.tcl - fred.

namespace import ::GL::*

# draw a triangle with vertices at (x1, y1), (x2, y2) 
# and (x3, y3) at z units away from the origin.
proc drawTriangle {x1 y1 x2 y2 x3 y3 z} {
    glBegin $::GL::GL_TRIANGLES
    glVertex3f $x1 $y1 $z
    glVertex3f $x2 $y2 $z
    glVertex3f $x3 $y3 $z
    glEnd
}

# draw a rectangular box with these outer x, y, and z values.
proc drawViewVolume {x1 x2 y1 y2 z1 z2} {
    glColor3f 1.0 1.0 1.0
    
    glBegin $::GL::GL_LINE_LOOP
    glVertex3f $x1 $y1 -$z1
    glVertex3f $x2 $y1 -$z1
    glVertex3f $x2 $y2 -$z1
    glVertex3f $x1 $y2 -$z1
    glEnd

    glBegin $::GL::GL_LINE_LOOP
    glVertex3f $x1 $y1 -$z2
    glVertex3f $x2 $y1 -$z2
    glVertex3f $x2 $y2 -$z2
    glVertex3f $x1 $y2 -$z2
    glEnd
    
    # 4 lines
    glBegin $::GL::GL_LINES
    glVertex3f $x1 $y1 -$z1
    glVertex3f $x1 $y1 -$z2
    glVertex3f $x1 $y2 -$z1
    glVertex3f $x1 $y2 -$z2
    glVertex3f $x2 $y1 -$z1
    glVertex3f $x2 $y1 -$z2
    glVertex3f $x2 $y2 -$z1
    glVertex3f $x2 $y2 -$z2
    glEnd
}

# drawScene draws 4 triangles and a wire frame
# which represents the viewing volume.
proc drawScene {} {
    glMatrixMode $::GL::GL_PROJECTION
    glLoadIdentity
    gluPerspective 40.0 [expr 4.0 / 3.0] 1.0 100.0
    
    glMatrixMode $::GL::GL_MODELVIEW
    glLoadIdentity
    gluLookAt 7.5 7.5 12.5 2.5 2.5 -5.0 0.0 1.0 0.0
    # green triangle
    glColor3f 0.0 1.0 0.0
    drawTriangle 2.0 2.0 3.0 2.0 2.5 3.0 -5.0
    # red triangle
    glColor3f 1.0 0.0 0.0
    drawTriangle 2.0 7.0 3.0 7.0 2.5 8.0 -5.0
    # yellow triangles
    glColor3f 1.0 1.0 0.0
    drawTriangle 2.0 2.0 3.0 2.0 2.5 3.0 0.0
    drawTriangle 2.0 2.0 3.0 2.0 2.5 3.0 -10.0
    drawViewVolume 0.0 5.0 0.0 5.0 0.0 10.0
}

# processHits prints out the contents of the selection array.
proc processHits {hits buffer} {
    puts "hits = $hits"
    binary scan $buffer i* buffer
    set ptr 0
    for {set i 0} {$i < $hits} {incr i} {
	set names [lindex $buffer $ptr]; incr ptr
	puts " number of names for hit = $names"
	puts -nonewline "  z1 is [expr double([lindex $buffer $ptr]) /\
		0x7fffffff];"; incr ptr;
	puts "  z2 is [expr double([lindex $buffer $ptr]) /\
		0x7fffffff];"; incr ptr;
	puts -nonewline "   the name is"
	# for each name
	for {set j 0} {$j < $names} {incr j} {
	    puts -nonewline " [lindex $buffer $ptr]"; incr ptr
	}
	puts ""
    }
}

# selectObjects "draws" the triangles in selection mode, 
# assigning names for the triangles. Note that the third
# and fourth triangles share one name, so that if either 
# or both triangles intersects the viewing/clipping volume, 
# only one hit will be registered.
proc selectObjects {} {
    glSelectBuffer 512 selectBuf
    glRenderMode $::GL::GL_SELECT
    
    glInitNames
    glPushName 0
    
    glPushMatrix
    glMatrixMode $::GL::GL_PROJECTION
    glLoadIdentity
    glOrtho 0.0 5.0 0.0 5.0 0.0 10.0
    glMatrixMode $::GL::GL_MODELVIEW
    glLoadIdentity 
    glLoadName 1
    drawTriangle 2.0 2.0 3.0 2.0 2.5 3.0 -5.0
    glLoadName 2
    drawTriangle 2.0 7.0 3.0 7.0 2.5 8.0 -5.0
    glLoadName 3
    drawTriangle 2.0 2.0 3.0 2.0 2.5 3.0 0.0
    drawTriangle 2.0 2.0 3.0 2.0 2.5 3.0 -10.0
    glPopMatrix 
    glFlush
    
    set hits [glRenderMode $::GL::GL_RENDER]
    processHits $hits $selectBuf
}

proc init {} {
    glEnable $::GL::GL_DEPTH_TEST
    glShadeModel $::GL::GL_FLAT
}

proc display {} {
    glClearColor 0.0 0.0 0.0 0.0
    glClear [expr $::GL::GL_COLOR_BUFFER_BIT | $::GL::GL_DEPTH_BUFFER_BIT]
    drawScene
    selectObjects
    glFlush
}

tom .gl -depth 1 -width 100 -height 100

init

bind .gl <Expose> {display}

bind .gl <Configure> {glViewport 0 0 %w %h}

pack .gl -fill both -expand 1
