# texture.tcl - fred.

namespace import ::GL::*

proc init {} {
    glClearColor 0.0 0.0 0.0 0.0
    glEnable $GL::GL_BLEND
    glBlendFunc $GL::GL_SRC_ALPHA $GL::GL_ONE_MINUS_SRC_ALPHA
    glEnable $GL::GL_DEPTH_TEST
    glDepthFunc $GL::GL_LESS
    glShadeModel $GL::GL_SMOOTH

    glPixelStorei $GL::GL_UNPACK_ALIGNMENT 1
    glTexParameteri $GL::GL_TEXTURE_2D $GL::GL_TEXTURE_WRAP_S $GL::GL_CLAMP
    glTexParameteri $GL::GL_TEXTURE_2D $GL::GL_TEXTURE_WRAP_T $GL::GL_CLAMP
    glTexParameteri $GL::GL_TEXTURE_2D $GL::GL_TEXTURE_MAG_FILTER $GL::GL_LINEAR
    glTexParameteri $GL::GL_TEXTURE_2D $GL::GL_TEXTURE_MIN_FILTER $GL::GL_LINEAR
    glTexEnvi $GL::GL_TEXTURE_ENV $GL::GL_TEXTURE_ENV_MODE $GL::GL_MODULATE
    glEnable $GL::GL_TEXTURE_2D

    texture
}

proc texture {} {
    set f [tk_getOpenFile]
    set info [exec mktex -info $f]
    if {$info == {}} return
    # make texture
    set width [lindex $info 0]
    set height [lindex $info 1]
    set tex [exec -keepnewline -- mktex -binary -rgba $f]
    if {$tex == {}} return
    puts "texture \"$f\": $width x $height"
    glTexImage2D -binary -variable $GL::GL_TEXTURE_2D 0 4\
	    $width $height 0 $GL::GL_RGBA $GL::GL_UNSIGNED_BYTE tex
    .gl expose
}

set dx 0
set dy 0
set bx 0
set by 0

proc display {} {
    glClear [expr $GL::GL_COLOR_BUFFER_BIT | $GL::GL_DEPTH_BUFFER_BIT]
    global dx dy
    
    glPushMatrix
    glTranslated -1.5 0.0 -5.0
    glRotatef $dx 0.0 1.0 0.0
    glRotatef $dy 1.0 0.0 0.0
    glBegin $GL::GL_QUADS
    glTexCoord2f 0.0 0.0; glVertex3f -1.0 -1.0 0.0
    glTexCoord2f 0.0 1.0; glVertex3f -1.0 1.0 0.0
    glTexCoord2f 1.0 1.0; glVertex3f 1.0 1.0 0.0
    glTexCoord2f 1.0 0.0; glVertex3f 1.0 -1.0 0.0
    glEnd
    glPopMatrix
    
    glPushMatrix
    glTranslated 1.5 0.0 -5.0
    glRotatef $dx 0.0 1.0 0.0
    glRotatef $dy 1.0 0.0 0.0
    set q [gluNewQuadric]
    gluQuadricTexture $q 1
    gluSphere $q 1 32 32
    gluDeleteQuadric $q
    glPopMatrix
        
    .gl swapbuffers
}

tom .gl -width 200 -height 200 -depth 1 -doublebuffer 1

init

bind .gl <Expose> {display}

bind .gl <Configure> {
    glViewport 0 0 %w %h
    glMatrixMode $GL::GL_PROJECTION
    glLoadIdentity
    gluPerspective 60.0 [expr double(%w) / double(%h)] 1.0 30.0
    glMatrixMode $GL::GL_MODELVIEW
    glLoadIdentity
}

bind .gl <ButtonPress-1> {
    set x %x
    set y %y
}

bind .gl <ButtonRelease-1> {
    set bx $dx
    set by $dy
}

bind .gl <Button1-Motion> {
    set dx [expr round($bx + (%x - $x) / 2.0) %% 360]
    set dy [expr round($by + (%y - $y) / 2.0) %% 360]
    .gl expose
}

bind .gl <ButtonPress-2> {texture}

bind .gl <KeyPress> {
    switch -exact -- %K {
	t {texture}
	Escape {destroy .}
    }
}   

pack .gl -fill both -expand true
focus .gl